@@ -290,8 +290,15 @@ void sys_set_priority(int higher)
     p3 = (higher ? p2 - 1 : p2 - 3);
 #endif
     par.sched_priority = p3;
-    if (sched_setscheduler(0,SCHED_FIFO,&par) != -1)
-       fprintf(stderr, "priority %d scheduling enabled.\n", p3);
+    if (sched_setscheduler(0,SCHED_FIFO,&par) < 0)
+    {
+        if (!higher)
+            post("priority %d scheduling failed; running at normal priority",
+                p3);
+        else fprintf(stderr, "priority %d scheduling failed.\n", p3);
+    }
+    else if (!higher && sys_verbose)
+        post("priority %d scheduling enabled.\n", p3);
 #endif
 
 #ifdef REALLY_POSIX_MEMLOCK /* this doesn't work on Fedora 4, for example. */
@@ -807,7 +814,13 @@ int sys_pollgui(void)
     return (sys_domicrosleep(0, 1) || sys_poll_togui());
 }
 
-
+void sys_init_fdpoll(void)
+{
+    /* create an empty FD poll list */
+    sys_fdpoll = (t_fdpoll *)t_getbytes(0);
+    sys_nfdpoll = 0;
+    inbinbuf = binbuf_new();
+}
 
 /* --------------------- starting up the GUI connection ------------- */
 
@@ -848,11 +861,8 @@ int sys_startgui(const char *libdir)
     int stdinpipe[2];
     pid_t childpid;
 #endif /* _WIN32 */
-    /* create an empty FD poll list */
-    sys_fdpoll = (t_fdpoll *)t_getbytes(0);
-    sys_nfdpoll = 0;
-    inbinbuf = binbuf_new();
-
+    sys_init_fdpoll();
+    
 #if !defined(_WIN32) && !defined(__CYGWIN__)
     signal(SIGHUP, sys_huphandler);
     signal(SIGINT, sys_exithandler);
@@ -1057,10 +1067,14 @@ int sys_startgui(const char *libdir)
                         wish_paths[i], libdir, PDGUIDIR, portno);
             }
 #else /* __APPLE__ */
+            /* sprintf the wish command with needed environment variables.
+            For some reason the wish script fails if HOME isn't defined so
+            if necessary we put that in here too. */
             sprintf(cmdbuf,
-  "TCL_LIBRARY=\"%s/lib/tcl/library\" TK_LIBRARY=\"%s/lib/tk/library\" \
+  "TCL_LIBRARY=\"%s/lib/tcl/library\" TK_LIBRARY=\"%s/lib/tk/library\"%s \
   wish \"%s/" PDGUIDIR "/pd-gui.tcl\" %d\n",
-                 libdir, libdir, libdir, portno);
+                 libdir, libdir, (getenv("HOME") ? "" : " HOME=/tmp"),
+                    libdir, portno);
 #endif /* __APPLE__ */
             sys_guicmd = cmdbuf;
         }
@@ -1152,7 +1166,9 @@ int sys_startgui(const char *libdir)
             sys_hipriority = 0;
         }
     }
-
+    else if (sys_verbose)
+        post("not setting real-time priority");
+    
     if (sys_hipriority)
     {
             /* To prevent lockup, we fork off a watchdog process with
