@@ -483,6 +483,12 @@ void sys_pollmidiqueue( void)
 
 /******************** dialog window and device listing ********************/
 
+#define MAXNDEV 20
+#define DEVDESCSIZE 80
+
+#define DEVONSET 1  /* To agree with command line flags, normally start at 1 */
+
+
 #ifdef USEAPI_ALSA
 void midi_alsa_init( void);
 #endif
@@ -493,9 +499,10 @@ void midi_oss_init( void);
     /* last requested parameters */
 static int midi_nmidiindev;
 static int midi_midiindev[MAXMIDIINDEV];
+static char midi_indevnames[MAXMIDIINDEV * DEVDESCSIZE];
 static int midi_nmidioutdev;
 static int midi_midioutdev[MAXMIDIOUTDEV];
-
+static char midi_outdevnames[MAXMIDIINDEV * DEVDESCSIZE];
 
 void sys_get_midi_apis(char *buf)
 {
@@ -511,18 +518,28 @@ void sys_get_midi_apis(char *buf)
         /* then again, if only one API (or none) we don't offer any choice. */
     if (n < 2)
         strcpy(buf, "{}");
-    
 }
+
 void sys_get_midi_params(int *pnmidiindev, int *pmidiindev,
     int *pnmidioutdev, int *pmidioutdev)
 {
-    int i;
+    int i, devn;
     *pnmidiindev = midi_nmidiindev;
-    for (i = 0; i < MAXMIDIINDEV; i++)
-        pmidiindev[i] = midi_midiindev[i]; 
+    for (i = 0; i < midi_nmidiindev; i++)
+    {
+        if ((devn = sys_mididevnametonumber(0,
+            &midi_indevnames[i * DEVDESCSIZE])) >= 0)
+                pmidiindev[i] = devn;
+        else pmidiindev[i] = midi_midiindev[i]; 
+    }
     *pnmidioutdev = midi_nmidioutdev;
-    for (i = 0; i < MAXMIDIOUTDEV; i++)
-        pmidioutdev[i] = midi_midioutdev[i]; 
+    for (i = 0; i < midi_nmidioutdev; i++)
+    {
+        if ((devn = sys_mididevnametonumber(1,
+            &midi_outdevnames[i * DEVDESCSIZE])) >= 0)
+                pmidioutdev[i] = devn;
+        else pmidioutdev[i] = midi_midioutdev[i]; 
+    }
 }
 
 static void sys_save_midi_params(
@@ -531,11 +548,19 @@ static void sys_save_midi_params(
 {
     int i;
     midi_nmidiindev = nmidiindev;
-    for (i = 0; i < MAXMIDIINDEV; i++)
-        midi_midiindev[i] = midiindev[i]; 
+    for (i = 0; i < nmidiindev; i++)
+    {
+        midi_midiindev[i] = midiindev[i];
+        sys_mididevnumbertoname(0, midiindev[i],
+            &midi_indevnames[i * DEVDESCSIZE], DEVDESCSIZE);
+    }
     midi_nmidioutdev = nmidioutdev;
-    for (i = 0; i < MAXMIDIOUTDEV; i++)
+    for (i = 0; i < nmidioutdev; i++)
+    {
         midi_midioutdev[i] = midioutdev[i]; 
+        sys_mididevnumbertoname(0, midioutdev[i],
+            &midi_outdevnames[i * DEVDESCSIZE], DEVDESCSIZE);
+    }
 }
 
 void sys_open_midi(int nmidiindev, int *midiindev,
@@ -572,23 +597,12 @@ void sys_reopen_midi( void)
     sys_open_midi(nmidiindev, midiindev, nmidioutdev, midioutdev, 1);
 }
 
-#define MAXNDEV 20
-#define DEVDESCSIZE 80
-
-#define DEVONSET 1  /* To agree with command line flags, normally start at 1 */
-
 void sys_listmididevs(void )
 {
     char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
     int nindevs = 0, noutdevs = 0, i;
 
-#ifdef USEAPI_ALSA
-    if (sys_midiapi == API_ALSA)
-      midi_alsa_getdevs(indevlist, &nindevs, outdevlist, &noutdevs,
-        MAXNDEV, DEVDESCSIZE);
-    else
-#endif /* ALSA */
-    midi_getdevs(indevlist, &nindevs, outdevlist, &noutdevs,
+    sys_get_midi_devs(indevlist, &nindevs, outdevlist, &noutdevs,
         MAXNDEV, DEVDESCSIZE);
 
     if (!nindevs)
@@ -617,32 +631,26 @@ void sys_set_midi_api(int which)
 }
 
 void glob_midi_properties(t_pd *dummy, t_floatarg flongform);
+void midi_alsa_setndevs(int in, int out);
 
 void glob_midi_setapi(void *dummy, t_floatarg f)
 {
     int newapi = f;
-    if (newapi == sys_midiapi)
-    {
-      //if (!midi_isopen())
-      //  s_reopen_midi();
-    }
-    else
+    if (newapi != sys_midiapi)
     {
 #ifdef USEAPI_ALSA
-      if (sys_midiapi == API_ALSA)
-          sys_alsa_close_midi();
-      else
+        if (sys_midiapi == API_ALSA)
+            sys_alsa_close_midi();
+        else
 #endif
-          sys_close_midi();
-      sys_midiapi = newapi;
-      /* bash device params back to default */
-      midi_nmidiindev = midi_nmidioutdev = 1;
-      //midi_midiindev[0] = midi_midioutdev[0] = DEFAULTMIDIDEV;
-      //midi_midichindev[0] = midi_midichoutdev[0] = SYS_DEFAULTCH;
-      sys_reopen_midi();
+              sys_close_midi();
+        sys_midiapi = newapi;
+        sys_reopen_midi();
     }
-
-    glob_midi_properties(0, 0);
+#ifdef USEAPI_ALSA
+    midi_alsa_setndevs(midi_nmidiindev, midi_nmidioutdev);
+#endif
+    glob_midi_properties(0, (midi_nmidiindev > 1 || midi_nmidioutdev > 1));
 }
 
 extern t_class *glob_pdobject;
@@ -654,14 +662,16 @@ void glob_midi_properties(t_pd *dummy, t_floatarg flongform)
         /* these are the devices you're using: */
     int nindev, midiindev[MAXMIDIINDEV];
     int noutdev, midioutdev[MAXMIDIOUTDEV];
-    int midiindev1, midiindev2, midiindev3, midiindev4,
-        midioutdev1, midioutdev2, midioutdev3, midioutdev4;
+    int midiindev1, midiindev2, midiindev3, midiindev4, midiindev5,
+        midiindev6, midiindev7, midiindev8, midiindev9,
+        midioutdev1, midioutdev2, midioutdev3, midioutdev4, midioutdev5,
+        midioutdev6, midioutdev7, midioutdev8, midioutdev9;
 
         /* these are all the devices on your system: */
     char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
     int nindevs = 0, noutdevs = 0, i;
 
-    midi_getdevs(indevlist, &nindevs, outdevlist, &noutdevs,
+    sys_get_midi_devs(indevlist, &nindevs, outdevlist, &noutdevs,
         MAXNDEV, DEVDESCSIZE);
 
     sys_gui("global midi_indevlist; set midi_indevlist {none}\n");
@@ -683,10 +693,20 @@ void glob_midi_properties(t_pd *dummy, t_floatarg flongform)
     midiindev2 = (nindev > 1 &&  midiindev[1]>= 0 ? midiindev[1]+1 : 0);
     midiindev3 = (nindev > 2 &&  midiindev[2]>= 0 ? midiindev[2]+1 : 0);
     midiindev4 = (nindev > 3 &&  midiindev[3]>= 0 ? midiindev[3]+1 : 0);
-    midioutdev1 = (noutdev > 0 && midioutdev[0]>=0 ? midioutdev[0]+1 : 0);  
-    midioutdev2 = (noutdev > 1 && midioutdev[1]>=0 ? midioutdev[1]+1 : 0);  
-    midioutdev3 = (noutdev > 2 && midioutdev[2]>=0 ? midioutdev[2]+1 : 0);  
-    midioutdev4 = (noutdev > 3 && midioutdev[3]>=0 ? midioutdev[3]+1 : 0);  
+    midiindev5 = (nindev > 4 &&  midiindev[4]>= 0 ? midiindev[4]+1 : 0);
+    midiindev6 = (nindev > 5 &&  midiindev[5]>= 0 ? midiindev[5]+1 : 0);
+    midiindev7 = (nindev > 6 &&  midiindev[6]>= 0 ? midiindev[6]+1 : 0);
+    midiindev8 = (nindev > 7 &&  midiindev[7]>= 0 ? midiindev[7]+1 : 0);
+    midiindev9 = (nindev > 8 &&  midiindev[8]>= 0 ? midiindev[8]+1 : 0);
+    midioutdev1 = (noutdev > 0 && midioutdev[0]>= 0 ? midioutdev[0]+1 : 0); 
+    midioutdev2 = (noutdev > 1 && midioutdev[1]>= 0 ? midioutdev[1]+1 : 0); 
+    midioutdev3 = (noutdev > 2 && midioutdev[2]>= 0 ? midioutdev[2]+1 : 0); 
+    midioutdev4 = (noutdev > 3 && midioutdev[3]>= 0 ? midioutdev[3]+1 : 0); 
+    midioutdev5 = (noutdev > 4 && midioutdev[4]>= 0 ? midioutdev[4]+1 : 0);
+    midioutdev6 = (noutdev > 5 && midioutdev[5]>= 0 ? midioutdev[5]+1 : 0);
+    midioutdev7 = (noutdev > 6 && midioutdev[6]>= 0 ? midioutdev[6]+1 : 0);
+    midioutdev8 = (noutdev > 7 && midioutdev[7]>= 0 ? midioutdev[7]+1 : 0);
+    midioutdev9 = (noutdev > 8 && midioutdev[8]>= 0 ? midioutdev[8]+1 : 0);
 
 #ifdef USEAPI_ALSA
       if (sys_midiapi == API_ALSA)
@@ -701,10 +721,13 @@ void glob_midi_properties(t_pd *dummy, t_floatarg flongform)
 #endif
     sprintf(buf,
 "pdtk_midi_dialog %%s \
-%d %d %d %d %d %d %d %d \
+%d %d %d %d %d %d %d %d %d \
+%d %d %d %d %d %d %d %d %d \
 %d\n",
-        midiindev1, midiindev2, midiindev3, midiindev4, 
-        midioutdev1, midioutdev2, midioutdev3, midioutdev4,
+        midiindev1, midiindev2, midiindev3, midiindev4, midiindev5,
+        midiindev6, midiindev7, midiindev8, midiindev9, 
+        midioutdev1, midioutdev2, midioutdev3, midioutdev4, midioutdev5,
+        midioutdev6, midioutdev7, midioutdev8, midioutdev9,
         (flongform != 0));
 
     gfxstub_deleteforkey(0);
@@ -717,16 +740,16 @@ void glob_midi_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv)
     int nmidiindev, midiindev[MAXMIDIINDEV];
     int nmidioutdev, midioutdev[MAXMIDIOUTDEV];
     int i, nindev, noutdev;
-    int newmidiindev[4], newmidioutdev[4];
+    int newmidiindev[9], newmidioutdev[9];
     int alsadevin, alsadevout;
 
-    for (i = 0; i < 4; i++)
+    for (i = 0; i < 9; i++)
     {
         newmidiindev[i] = atom_getintarg(i, argc, argv);
-        newmidioutdev[i] = atom_getintarg(i+4, argc, argv);
+        newmidioutdev[i] = atom_getintarg(i+9, argc, argv);
     }
 
-    for (i = 0, nindev = 0; i < 4; i++)
+    for (i = 0, nindev = 0; i < 9; i++)
     {
         if (newmidiindev[i] > 0)
         {
@@ -734,7 +757,7 @@ void glob_midi_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv)
             nindev++;
         }
     }
-    for (i = 0, noutdev = 0; i < 4; i++)
+    for (i = 0, noutdev = 0; i < 9; i++)
     {
         if (newmidioutdev[i] > 0)
         {
@@ -742,21 +765,37 @@ void glob_midi_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv)
             noutdev++;
         }
     }
-    alsadevin = atom_getintarg(8, argc, argv);
-    alsadevout = atom_getintarg(9, argc, argv);
-        
+    alsadevin = atom_getintarg(18, argc, argv);
+    alsadevout = atom_getintarg(19, argc, argv);
+#ifdef USEAPI_ALSA
+            /* invent a story so that saving/recalling "settings" will
+            be able to restore the number of devices.  ALSA MIDI handling
+            uses its own set of variables.  LATER figure out how to get
+            this to work coherently */
+    if (sys_midiapi == API_ALSA)
+    {
+        nindev = alsadevin;
+        noutdev = alsadevout;
+        for (i = 0; i < nindev; i++)
+            newmidiindev[i] = i;
+        for (i = 0; i < noutdev; i++)
+            newmidioutdev[i] = i;
+    }
+#endif
+    sys_save_midi_params(nindev, newmidiindev,
+        noutdev, newmidioutdev);
 #ifdef USEAPI_ALSA
     if (sys_midiapi == API_ALSA)
-      {
+    {
         sys_alsa_close_midi();
         sys_open_midi(alsadevin, newmidiindev, alsadevout, newmidioutdev, 1);
-      }
+    }
     else
 #endif
-      {
+    {
         sys_close_midi();
         sys_open_midi(nindev, newmidiindev, noutdev, newmidioutdev, 1);
-      }
+    }
 
 }
 
@@ -773,3 +812,62 @@ void sys_get_midi_devs(char *indevlist, int *nindevs,
 #endif /* ALSA */
   midi_getdevs(indevlist, nindevs, outdevlist, noutdevs, maxndevs, devdescsize);
 }
+
+/* convert a device name to a (1-based) device number.  (Output device if
+'output' parameter is true, otherwise input device).  Negative on failure. */
+
+int sys_mididevnametonumber(int output, const char *name)
+{
+    char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
+    int nindevs = 0, noutdevs = 0, i;
+
+    sys_get_midi_devs(indevlist, &nindevs, outdevlist, &noutdevs,
+        MAXNDEV, DEVDESCSIZE);
+
+    if (output)
+    {
+        for (i = 0; i < noutdevs; i++)
+        {
+            unsigned int comp = strlen(name);
+            if (comp > strlen(outdevlist + i * DEVDESCSIZE))
+                comp = strlen(outdevlist + i * DEVDESCSIZE);
+            if (!strncmp(name, outdevlist + i * DEVDESCSIZE, comp))
+                return (i);
+        }
+    }
+    else
+    {
+        for (i = 0; i < nindevs; i++)
+        {
+            unsigned int comp = strlen(name);
+            if (comp > strlen(indevlist + i * DEVDESCSIZE))
+                comp = strlen(indevlist + i * DEVDESCSIZE);
+            if (!strncmp(name, indevlist + i * DEVDESCSIZE, comp))
+                return (i);
+        }
+    }
+    return (-1);
+}
+
+/* convert a (1-based) device number to a device name.  (Output device if
+'output' parameter is true, otherwise input device).  Empty string on failure.
+*/
+
+void sys_mididevnumbertoname(int output, int devno, char *name, int namesize)
+{
+    char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
+    int nindevs = 0, noutdevs = 0, i;
+    if (devno < 0)
+    {
+        *name = 0;
+        return;
+    }
+    sys_get_midi_devs(indevlist, &nindevs, outdevlist, &noutdevs,
+        MAXNDEV, DEVDESCSIZE);
+    if (output && (devno < noutdevs))
+        strncpy(name, outdevlist + devno * DEVDESCSIZE, namesize);
+    else if (!output && (devno < nindevs))
+        strncpy(name, indevlist + devno * DEVDESCSIZE, namesize);
+    else *name = 0;
+    name[namesize-1] = 0;
+}
